﻿using Dapper;
using Microsoft.Extensions.Logging;
using Microsoft.Extensions.Options;
using MySql.Data.MySqlClient;
using Spider.Core.Caching;
using System;
using System.Collections.Generic;
using System.Threading.Tasks;

namespace Spider.Core.DataProvider
{
    public class ProductionDataProvider : IDataProvider<Production>
    {
        private readonly MySqlOptions _options;
        private readonly ILogger _logger;

        public ProductionDataProvider(
            IOptions<MySqlOptions> options,
            ILogger<ProductionDataProvider> logger)
        {
            _logger = logger;
            _options = options.Value;
        }

        public async Task<IEnumerable<Production>> GetAsync(DataCollectContext context)
        {
            var sql = $"SELECT `ProductionID` `Id`,`ShowProductionName` `ShortName` FROM `tproduction` WHERE `ProductionID` > @LargeBatch LIMIT @BatchSize;";

            var batchSize = context.SmallBatchSize;
            var largeBatchLastId = context.CheckPointId + (context.LargeBatch - 1) * batchSize;

            try
            {
                using (var connection = new MySqlConnection(_options.ConnectionString))
                {
                    var result = await connection.QueryAsync<Production>(sql, new { LargeBatch = largeBatchLastId, BatchSize = batchSize });
                    return result;
                }
            }
            catch (Exception e)
            {
                _logger.LogError($"{nameof(GetAsync)}[batchSize: {batchSize}, MySqlOptions: {_options}, sql: {sql}] has a unknown exception: {e}");

                return default;
            }
        }

        public async Task<int> GetCheckPointIdAsync()
        {
            var sql = $"SELECT `ProductionID` FROM `tproduction` ORDER BY `ProductionID` LIMIT 1;";

            try
            {
                using (var connection = new MySqlConnection(_options.ConnectionString))
                {
                    var result = await connection.ExecuteScalarAsync<int>(sql);
                    return result;
                }
            }
            catch (Exception e)
            {
                _logger.LogError($"{nameof(GetCheckPointIdAsync)}[MySqlOptions: {_options}, sql: {sql}]has a unknown exception: {e}");

                return default;
            }
        }

        public async Task<int> GetCountAsync()
        {
            var sql = $"SELECT count(*) FROM `tproduction`;";

            try
            {
                using (var connection = new MySqlConnection(_options.ConnectionString))
                {
                    var result = await connection.ExecuteScalarAsync<int>(sql);
                    return result;
                }
            }
            catch (Exception e)
            {
                _logger.LogError($"{nameof(GetCountAsync)}[MySqlOptions: {_options}, sql: {sql}]has a unknown exception: {e}");

                return default;
            }
        }
    }
}
