﻿using Dapper;
using Microsoft.Extensions.Logging;
using Microsoft.Extensions.Options;
using MySql.Data.MySqlClient;
using Spider.Core.Caching;
using System;
using System.Collections.Generic;
using System.Threading.Tasks;

namespace Spider.Core.DataProvider
{
    public class UserDataProvider : IDataProvider<User>
    {
        private readonly MySqlOptions _options;
        private readonly ILogger _logger;

        public UserDataProvider(
            IOptions<MySqlOptions> options,
            ILogger<UserDataProvider> logger)
        {
            _logger = logger;
            _options = options.Value;
        }

        [CacheReturnValue]
        public async Task<IEnumerable<User>> GetAsync(DataCollectContext context)
        {
            var sql = $"SELECT `UserId` `Id`,`UserName` `Name`,`avatar` `Avatar` FROM `tuser` WHERE `UserId`>@LargeBatch LIMIT @BatchSize;";

            var batchSize = context.SmallBatchSize;
            var largeBatchLastId = context.UserCheckPointId + (context.LargeBatch - 1) * batchSize;

            try
            {
                using (var connection = new MySqlConnection(_options.ConnectionString))
                {
                    var result = await connection.QueryAsync<User>(sql, new { LargeBatch = largeBatchLastId, BatchSize = batchSize });
                    return result;
                }
            }
            catch (Exception e)
            {
                _logger.LogError($"{nameof(GetAsync)}[batchSize: {batchSize}, MySqlOptions: {_options}, sql: {sql}] has a unknown exception: {e}");

                return default;
            }
        }

        [CacheReturnValue]
        public async Task<int> GetCheckPointIdAsync()
        {
            var sql = $"SELECT `UserId` FROM `tuser` ORDER BY `UserId` LIMIT 1;";

            try
            {
                using (var connection = new MySqlConnection(_options.ConnectionString))
                {
                    var result = await connection.ExecuteScalarAsync<int>(sql);
                    return result;
                }
            }
            catch (Exception e)
            {
                _logger.LogError($"{nameof(GetCheckPointIdAsync)}[MySqlOptions: {_options}, sql: {sql}]has a unknown exception: {e}");

                return default;
            }
        }

        public Task<int> GetCountAsync()
        {
            throw new NotSupportedException();
        }
    }
}
