﻿using Microsoft.Extensions.Logging;
using Microsoft.Extensions.Options;
using OpenQA.Selenium;
using OpenQA.Selenium.Support.UI;
using Spider.Core.DataProvider;
using Spider.Infrastructure.Pos;
using Spider.Services;
using System;
using System.Collections.Generic;
using System.Linq;

namespace Spider.Core.Pipeline
{
    public class Extractor : IExtractable
    {
        private readonly IWebDriver _driver;
        private readonly DataCollectorOptions _options;
        private readonly ILogger _logger;

        public Extractor(
            IWebDriverProvider provider,
            IOptions<DataCollectorOptions> options,
            ILogger<Extractor> logger)
        {
            _logger = logger;
            _options = options.Value;
            _driver = provider.Get(Browsers.Chrome);
            Initialize();
        }

        private void Initialize()
        {
            _driver.Navigate().GoToUrl(_options.TargetUrl);
        }

        public List<PayLoad> Extract(Production production)
        {
            if (production == null)
                return null;

            _logger.LogInformation($"Production[{production.Id}] begin crawling data...");

            try
            {
                var payloads = DataCollect(production);
                if (payloads == null || !payloads.Any())
                    _logger.LogWarning($"Production[{production.Id}] has not questions & answers data.");
                else
                    return payloads;
            }
            catch (Exception e)
            {
                _logger.LogError($"Crawling data has a unknown exception: {e}");
            }

            _logger.LogInformation($"Production[{production.Id}] Extracted.");

            return null;
        }

        private List<PayLoad> DataCollect(Production production)
        {
            // 等待页面加载
            _driver.FindElement(By.Id("searchInput")).SendKeys(production.ShortName + Keys.Enter);

            // 1. Find the crawl target.
            // 等待搜索结果。
            var searchItem = By.XPath($"//ul[@class='clearfix js_seachResultList'][{_options.SearchResultIndex}]/li[{_options.SearchResultIndex}]/a");
            var wait = new WebDriverWait(_driver, TimeSpan.FromSeconds(_options.SearchTimeOutSeconds));
            wait.Until(wd => wd.FindElement(searchItem)).Click();

            // 问答选项卡
            //var path = By.XPath(@"//ul[@class='compTab']/li[3]");
            //_driver.FindElement(path).Click();

            // 2. Crawling data.
            // 问答列表(All)
            var path = By.XPath(@"//ul[@id='js-faqFilter']/li[1]");
            var count = _driver.FindElement(path).GetAttribute("data-total");
            if (!int.TryParse(count, out var all) || all <= 0)
                return null;

            return CrawlingData(production);
        }

        private List<PayLoad> CrawlingData(Production production)
        {
            const string path = @"//div[@class='goodsFAQ_panelList']/div[@class='goodsFAQ_panel js-itemPanelCustomerFAQ'][1]//dl[@class='goodsFAQ_customerGroup']";
            var elements = _driver.FindElements(By.XPath(path));
            if (elements == null || !elements.Any())
                return null;

            var payloads = new List<PayLoad>();
            foreach (var element in elements)
            {
                var payload = PayloadParse(production, element);
                if (payload != null)
                    payloads.Add(payload);
            }

            return payloads;
        }

        private PayLoad PayloadParse(Production production, IWebElement element)
        {
            try
            {
                // 问题
                var path = By.XPath(@".//dt[@class='goodsFAQ_customerTitle']");
                var questionEle = element.FindElement(path);
                if (questionEle == null) return null;

                path = By.XPath(@".//h4[@class='goodsFAQ_customerQuestion']");
                var qContent = questionEle.FindElement(path)?.Text;

                path = By.XPath(@".//p[@class='goodsFAQ_customerTime']");
                var customerTime = questionEle.FindElement(path)?.Text;
                var question = QuestionParse(customerTime);

                // 回答
                path = By.XPath(@".//dd[@class='goodsFAQ_customerAnswer']");
                var answerEle = element.FindElement(path);

                path = By.XPath(@".//div[@class='goodsFAQ_customerCont']/p[2]");
                var aContent = answerEle?.FindElement(path)?.Text;

                // 问题必须爬取，答案可以为空。
                var payload = new PayLoad()
                {
                    Question = new Question()
                    {
                        ProductionId = production.Id,
                        ProductionName = production.ShortName,
                        Content = qContent,
                        QuestionerName = question.QuestionerName,
                        CreateTime = question.CreateTime
                    }
                };

                if (!string.IsNullOrEmpty(aContent))
                {
                    payload.Answer = new Answer()
                    {
                        Content = aContent
                    };
                }

                return payload;
            }
            catch (Exception e)
            {
                _logger.LogError($"{nameof(PayloadParse)} has a unknown exception: {e}");
                return null;
            }
        }

        private (string QuestionerName, DateTime? CreateTime) QuestionParse(string customerTime)
        {
            if (string.IsNullOrEmpty(customerTime))
                return (string.Empty, null);

            var length = customerTime.Length;
            var index = customerTime.IndexOf('-') - 3;
            var buffer = customerTime.AsSpan();
            var questioner = buffer.Slice(0, index).Trim();
            var time = buffer.Slice(index, length - index).Trim();

            return (questioner.ToString(), DateTime.Parse(time));
        }
    }
}
